@@ -6,9 +6,11 @@ use B::Generate;
 use B::Utils qw(walkallops_simple);
 use B qw(OPf_KIDS OPf_MOD OPf_PARENS OPf_WANT_SCALAR OPf_STACKED);
 use Attribute::Handlers;
-use Hook::Scope qw(POST);
-our $VERSION = "0.03_02";
+use B::Hooks::EndOfScope;
 
+our $VERSION = "0.03_03";
+
+our $DEBUG = 0;
 our %pads;
 our $state;
 our $old_op;
@@ -16,32 +18,33 @@ our %loaded;
 our $stash = '';
 our %register;
 
+sub dbgprint { print @_ if $DEBUG; }
 use optimizer "extend-c" => sub {
     my $op = shift;
-    POST(sub{$old_op = $op;()});
+    on_scope_end {$old_op = $op;()};
     return unless $op;
-    if($op->name eq 'nextstate') {
+    if ($op->name eq 'nextstate') {
 	$state = $op;
 	$stash = $state->stash->NAME;
-        # print $state->file . ":" . $state->line . "-" . $state->stash->NAME . "\n";
-
-        if($stash =~/^(optimize|B::|types$|float$|double$|int$|number$|^O$)/) {
-            #	print "Don't optimize ourself\n";
+        dbgprint $state->file . ":" . $state->line . "-" . $state->stash->NAME . "\n";
+        if ($stash =~/^(optimize|B::|types$|float$|double$|int$|number$|^O$)/) {
+            dbgprint "Don't optimize ourself\n";
             return;
         }
     }
 
-    # print "$op - " . $op->name . " - " . $op->next . " - " . ($op->next->can('name') ? $op->next->name : "") . "\n";
+    dbgprint ref($op)." - " . $op->name . " - " . ref($op->next) . " - " . 
+             ($op->next->can('name') ? $op->next->name : "null") . "\n";
     my $cv;
     eval {
 	$cv = $op->find_cv;
     };
-    if($@) {
+    if ($@) {
 	$@ =~s/\n//;
 	print "$@ in " . $state->file . ":" . $state->line . "\n";;
 	return;
     }
-    if($op->name eq 'const' &&
+    if ($op->name eq 'const' &&
        ref($op->sv) eq 'B::PV' && 
        $op->sv->sv eq 'attributes' &&
        $op->can('next') &&
@@ -60,15 +63,15 @@ use optimizer "extend-c" => sub {
 	
 	my $attribute = $op->next->next->next->next->next->sv->sv;
 	
-	if($attribute =~/^optimize\(\s*(.*)\s*\)/) {
-            # print "$attribute\n";
+	if ($attribute =~/^optimize\(\s*(.*)\s*\)/) {
+            #dbgprint "attr: $attribute\n";
 	    my @attributes = split /\s*,\s*/, $1;
-            # print "GOT " . join("-", @attributes) . "\n";
-
-	    if($op->next->next->name eq 'padsv') {
-		my $sv = (($cv->PADLIST->ARRAY)[0]->ARRAY)[$op->next->next->targ];
-		my $ref = $pads{$cv->ROOT->seq}->[$op->next->next->targ] = [$sv->sv(),{}];
-		for(@attributes) {
+            dbgprint "GOT " . join("-", @attributes) . "\n";
+	    my $opnn = $op->next->next;
+	    if ($opnn->name eq 'padsv') {
+		my $sv = (($cv->PADLIST->ARRAY)[0]->ARRAY)[$opnn->targ];
+		my $ref = $pads{$cv->ROOT->seq}->[$opnn->targ] = [$sv->sv(),{}];
+		for (@attributes) {
 		    $ref->[1]{$_}++;
 		    unless($loaded{$_}) {
 			require "optimize/$_.pm";			
@@ -80,14 +83,14 @@ use optimizer "extend-c" => sub {
     }
 
     for (values %loaded) {	
-        # print "Calling $_\n";
+        dbgprint "Calling $_\n";
 	$_->check($op);
-        # print "Called $_\n";
+        dbgprint "Called $_\n";
     }
     # calling types
-    if(exists($register{$stash})) {
+    if (exists($register{$stash})) {
 	for my $callback (values %{$register{$stash}}) {
-	    if($callback) {
+	    if ($callback) {
 		$callback->($op);
 	    }
 	}
@@ -127,19 +130,19 @@ optimize - Pragma for hinting optimizations on variables
     my $int : optimize(int);
     $int = 1.5;
     $int += 1;
-    if($int == 2) { print "$int is integerized" }
+    if ($int == 2) { print "$int is integerized" }
 
-    #Following will call this callback with the op
-    #as the argument if you are in the specified package
-    #see types.pm how it is used from import and unimport
+    # Following will call this callback with the op
+    # as the argument if you are in the specified package.
+    # See L<types> how it is used from import and unimport.
     optimize->register(\&callback, $package);
 
-    #and reverse it
+    # and reverse it
     optimize->unregister($package);
 
 =head1 DESCRIPTION
 
-optimize allows you to use attributes to turn on optimizations.
+B<optimize> allows you to use attributes to turn on optimizations.
 It works as a framework for different optimizations.
 
 =head1 BUGS
@@ -149,7 +152,7 @@ different optimizations will be in a different state of readyness
 
 =head1 AUTHOR
 
-Arthur Bergman E<lt>abergman at cpan.orgE<gt>
+Artur Bergman E<lt>abergman at cpan.orgE<gt>
 
 =head1 SEE ALSO
 
